// Main.qml
import QtQuick 2.15
import QtQuick.Window 2.15
import QtQuick.Controls 2.15
import org.ukui.desktop 1.0
import org.ukui.quick.items 1.0
import org.ukui.quick.platform 1.0

import "./autoArrangeControler.js" as AutoArrangeControler
import "." as ComponentsManager

Rectangle {
    id : window
    objectName: "rootRectangle"
    anchors.fill: parent
    property int number: Screen.serialNumber
    visible: true
    color: "transparent"
    property int spaceId: -1
    property bool acceptDrop: true
    property bool isDraging: false
    property bool ismultifilemode: false
    property bool isBoxSelection: false
    property point pressPos: Qt.point(0, 0)
    property bool sorting: false
    property bool initDesktop: false
    signal dropRequest(var urls)
    signal addItem();

    BackgroundItem {
        id: backgroundItem
        anchors.fill: parent
        backImagePath: backgroundManager.getBackImagePath()
        frontImagePath: backgroundManager.getFrontImagePath()
        backgroundOption: backgroundManager.getBackgroundOption()
        usePureColor: backgroundManager.getUsePureColor()
        bgColor: backgroundManager.getColor()
        screenGeometry: Qt.rect(Screen.virtualX, Screen.virtualY, Screen.width, Screen.height)
    }

    Connections {
        target: backgroundManager
        function onBackgroundImageChanged() {
            backgroundItem.backImagePath = backgroundManager.getBackImagePath()
            backgroundItem.frontImagePath = backgroundManager.getFrontImagePath()
        }
        function onBackgroundOptionChanged() {
            backgroundItem.backgroundOption = backgroundManager.getBackgroundOption()
        }
        function onUsePureColorChanged() {
            backgroundItem.usePureColor = backgroundManager.getUsePureColor()
        }
        function onColorChanged() {
            backgroundItem.bgColor = backgroundManager.getColor()
        }
    }

    Connections {
        target: Qt.application
        function onScreensChanged() {backgroundItem.handleSpanned()}
    }

    SelectionModel {
        id: selectionModel
        targetView: desktopIconView
        function onClearAllStatus() {
            desktopIconView.closeEdit()
        }
    }

    ShortcutManager {
        id: shortcutManager
    }

    Connections {
        target: EventBus
        function onFocusChanged(screenId) {
            console.log("Window2 EventBus instance:", EventBus);
            var ss = desktopIconView.currentScreentId
            if (screenId !== desktopIconView.currentScreentId) {
                selectionModel.clearSelection()
            }
            console.log("Focus changed to screen", screenId);
        }
    }
    function layoutchange(){
        window.isDraging = false
        window.sorting = true
//        for (var i = 0; i < desktopIconView.count; i++) {
//            var item = desktopIconView.itemAt(i);
//            if (item) {
//                desktopIconView.addItemPosition(item.uri, item.pos.x, item.pos.y);
//                var tmpX = desktopIconView.transtogridposx(item.pos.x*desktopIconView.gridWidth,desktopIconView.gridWidth)
//                var tmpY = desktopIconView.transtogridposy(item.pos.y*desktopIconView.gridHeight,desktopIconView.gridHeight)
//                var tmp = Qt.point(tmpX, tmpY)
//                // note: 这里不直接设置item的x和y，避免behavior的过渡效果失效
//                //item.x = desktopIconView.transtogridposx(item.pos.x*desktopIconView.gridWidth,desktopIconView.gridWidth)
//                //item.y = desktopIconView.transtogridposy(item.pos.y*desktopIconView.gridHeight,desktopIconView.gridHeight)
//            }
//            item.startSortanima()
//            var uri = item.uri
//            desktopIconView.preitemsPos[item.uri] = Qt.point(item.pos.x, item.pos.y)
//        }
    }
    ListModel {
        id: dragitemModel
    }

    Item {
        id: dragIconView
        //anchors.fill: parent
        visible: false
        width: bounds.width
        height: bounds.height
        clip: true  // 启用裁剪
        z: 100

        // 动态计算边界
        property rect bounds: calculateContentBounds()
        property real offsetToBoundsTopLeftX: 0
        property real offsetToBoundsTopLeftY: 0

        Drag.supportedActions: Qt.CopyAction | Qt.MoveAction
        Drag.dragType: Drag.Automatic
        Drag.onDragStarted: {
            console.log("item drag start")
            isDraging = true
        }

        Drag.onDragFinished: {
            console.log("item drag finish======")
            isDraging = false
        }

        function calculateContentBounds() {
            var minX = 0, minY = 0;
            var maxX = 0, maxY = 0;
            if (dragitemModel.count > 0) {
                minX = dragitemModel.get(0).x
                minY = dragitemModel.get(0).y
            }
            for (var i = 0; i < dragitemModel.count; i++) {
                var item = dragitemModel.get(i);
                minX = Math.min(minX, item.x);
                minY = Math.min(minY, item.y);
                maxX = Math.max(maxX, item.x);
                maxY = Math.max(maxY, item.y);
                console.log("minX, minY, maxX - minX, maxY - minY", minX, minY, maxX - minX, maxY - minY,item.x,item.y,dragitemModel.count)
            }
            return Qt.rect(minX, minY, maxX - minX + desktopIconView.gridWidth , maxY - minY + desktopIconView.gridHeight );
        }

        Repeater {
            model: dragitemModel
            delegate: Rectangle {
                id: deleitem
                x: model.x - parent.bounds.x
                y: model.y - parent.bounds.y
                width: desktopIconView.gridWidth - 10
                height: desktopIconView.gridHeight - 10
                color: "transparent"
                DelegateIcon {
                    id: dragviewIconRect
                    width:  controller.iconSize.width
                    height:  controller.iconSize.height
                    icon : controller.icon(model.uri)
                    emblems: model.emblems
                    emblemSize: controller.emblemSize
                }
                DelegateText {
                    id: dragdelegateText
                    anchors.horizontalCenter: dragviewIconRect.horizontalCenter
                    anchors.top: dragviewIconRect.bottom
                    display: model.display
                    editing: false  // 是否处于编辑状态
                }
            }
        }
    }

    //绘制网格虚线辅助  可注释
    //    Repeater {
    //        model: Math.ceil(parent.height / desktopIconView.gridHeight)
    //        delegate: Rectangle {
    //            width: parent.width
    //            height: 1
    //            y: index * desktopIconView.gridHeight
    //            color: "transparent"
    //            border.color: Qt.rgba(0.5, 0.5, 0.5, 0.3)
    //            border.width: 1
    //            layer.enabled: true
    //            //layer.effect: DashLineEffect {} // 需要自定义DashLineEffect
    //        }
    //    }

    //    // 垂直虚线
    //    Repeater {
    //        model: Math.ceil(parent.width / desktopIconView.gridWidth)
    //        delegate: Rectangle {
    //            width: 1
    //            height: parent.height
    //            x: index *  desktopIconView.gridWidth
    //            color: "transparent"
    //            border.color: Qt.rgba(0.5, 0.5, 0.5, 0.3)
    //            border.width: 1
    //            layer.enabled: true
    //            //.effect: DashLineEffect {}
    //        }
    //    }

    Item {
        id: contentArea
        anchors.fill: parent
        anchors.topMargin: globalSettings.panelMarginTop
        anchors.rightMargin: globalSettings.panelMarginRight
        anchors.bottomMargin: globalSettings.panelMarginBottom
        anchors.leftMargin: globalSettings.panelMarginLeft  // 关键：留出边框宽度

        // 拖拽插入指示线
        Rectangle {
            id: dropIndicator
            visible: false
            width: desktopIconView.gridWidth
            height: 4
            color: "white"
            z: -10
        }

        Rectangle {
            id: displayTooltips
            width: name.implicitWidth + 10
            height: name.implicitHeight + 10
            radius: 3

            visible: false
            Text {
                id: name
                text: ""
                anchors.centerIn: parent
                textFormat: Text.RichText
            }
            border.color: "lightgray"
            border.width: 1

            property alias tooltipText: name.text
        }
        Repeater {
            id: desktopIconView
            model: myProxyModel
            property int gridWidth: controller.gridSize.width
            property int gridHeight: controller.gridSize.height
            property int currentEditIndex: -1 // 当前编辑项的索引
            property int initDataCount: 0
            property int currentScreentId: model.id()
            property bool autoArrange: globalSettings.useAutoLayout()
            //property bool initDesktop: false
            property var resolutionItemPos: []
            property var itemsPos: []
            property var urilist: []
            property var preitemsPos:({})
            property point delegateGlobalPos
            property alias isMultifileMode: window.ismultifilemode

            function transtogridposx(x,width){
                var centerX = x + width/2;
                // 计算最近网格索引
                var gridX = Math.round(centerX /gridWidth);
                gridX = Math.abs(centerX -gridWidth*(gridX-0.5))<Math.abs(gridWidth*(gridX+0.5) -centerX)? gridX-1:gridX
                gridX = Math.max(0, gridX);
                return (gridX)*gridWidth
            }
            function transtogridposy(y,height){
                var centerY = y + height/2;
                // 计算最近网格索引
                var gridY = Math.round(centerY /gridHeight);
                gridY = Math.abs(centerY -gridHeight*(gridY-0.5))<Math.abs(gridHeight*(gridY+0.5) -centerY)? gridY-1:gridY
                gridY = Math.max(0, gridY);
                return gridY*gridHeight
            }
            function closeEdit() {
                console.log("closeEdit",desktopIconView.currentEditIndex);
                if (desktopIconView.currentEditIndex !== -1) {
                    var item = desktopIconView.itemAt(desktopIconView.currentEditIndex)
                    if (item && item.delegateText) {
                        item.delegateText.updateFileName();
                    }
                    desktopIconView.currentEditIndex = -1
                }
            }

            function indexAt(x, y) {
                console.log("获取index 通过x，y", x,  y)
                for (var i = 0; i < desktopIconView.count; ++i) {
                    var item = desktopIconView.itemAt(i)
                    if (x >= item.x && x <= item.x + desktopIconView.gridWidth &&
                            y >= item.y && y <= item.y + desktopIconView.gridHeight) {
                        console.log("获取索引", i, item.uri)
                        return i // 返回匹配的索引
                    }
                }
                return -1 // 未找到匹配项
            }

            function onUpdateDropIndicator(pos) {
                if (desktopIconView.autoArrange === false) {
                    return
                }

                console.log("current mouse pos:",pos.x, pos.y)
                var hoverIndex = desktopIconView.indexAt(pos.x, pos.y)
                console.log("hover index:",hoverIndex)
                if (hoverIndex >= 0) {
                    var hoverItem = desktopIconView.itemAt(hoverIndex)
                    var localPos = hoverItem.mapFromItem(desktopIconView, pos.x, pos.y)

                    if (localPos.y < hoverItem.height * 0.3) {
                        dropIndicator.y = hoverItem.y
                        dropIndicator.visible = true
                    }
                    else if (localPos.y > hoverItem.height * 0.7) {
                        dropIndicator.y = hoverItem.y + hoverItem.height
                        dropIndicator.visible = true
                    }
                    else {
                        dropIndicator.visible = false
                        return
                    }
                    dropIndicator.x = hoverItem.x
                } else {
                    dropIndicator.visible = false
                }
            }
            delegate: Rectangle {
                id: dele
                x: pos.x*desktopIconView.gridWidth
                y: pos.y*desktopIconView.gridHeight
                z: selected? parent.z + 1 : parent.z
                width: desktopIconView.gridWidth - 10
                height: selected && delegateText.showFullText ? Math.max(viewIconRect.height + delegateText.height + 15, desktopIconView.gridHeight - 10)  : desktopIconView.gridHeight - 10
                radius: themeStyle.radius
                color: selected && !delegateText.editing ? themeStyle.highlight() : hovered && !delegateText.editing ? themeStyle.midlight() : "transparent"
                focus: true
                activeFocusOnTab: true
                layer.samples: 4     // 抗锯齿
                property string uri: model.uri
                property string display: model.display
                property string tooltip: model.tooltip
                property point pos: model.position
                property bool selected: false
                property alias hovered: gridViewDelegateMouseArea.containsMouse
                property alias delegateText: delegateText  // 将子项暴露为属性
                property alias pressPos1: gridViewDelegateMouseArea.pressPos1
                property point delegateGlobalPos
                property bool isFromDesktopView: true
                property int itemIndex: index
                property int dragx: x
                property int dragtox: x
                property int dragy: y
                property int dragtoy: y
                property alias animItem: anim
                property alias animyItem: animy
                property alias xBehaviorObject: xTransBehavior
                property alias yBehaviorObject: yTransBehavior
                property var decoration: model ? model.decoration : ""
                property var emblems:  model ? model.emblems : ""
                function startSortanima() {
                    //animaresort.start()
                    //animaresorty.start()
                }
                function startinitanima() {
                    //initanim.start()
                }

                ParallelAnimation {
                    id: initAnimation
                    XAnimator {
                        target: dele
                        from: x - desktopIconView.gridWidth
                        to: x
                        duration: 300
                        easing.type: Easing.Bezier
                        easing.bezierCurve: [0.25, 0.1, 0.38, 1]
                    }
                    OpacityAnimator {
                        target: dele
                        //property: "opacity"
                        from: 0.0
                        to: 1.0
                        duration: 300
                        easing.type: Easing.Bezier
                        easing.bezierCurve: [0.25, 0.1, 0.38, 1]
                    }
                }

                Connections {
                    target: window
                    function onInitDesktopChanged() {
                        initAnimation.start()
                    }
                }
                //            ParallelAnimation {
                //                id: initanim
                //                NumberAnimation {
                //                    target: dele
                //                    property: "x"
                //                    from: dele.x - desktopIconView.gridWidth
                //                    to: dele.x
                //                    duration: 300
                //                    easing.type: Easing.Bezier
                //                    easing.bezierCurve: [0.25, 0.1, 0.38, 1]
                //                }
                //                NumberAnimation {
                //                    target: dele
                //                    property: "opacity"
                //                    from: 0.0
                //                    to: 1.0
                //                    duration: 300
                //                    easing.type: Easing.Bezier
                //                    easing.bezierCurve: [0.25, 0.1, 0.38, 1]

                //                    onStopped: {
                //                        desktopIconView.initDataCount = desktopIconView.initDataCount + 1
                //                        desktopIconView.initDesktop = desktopIconView.initDataCount === desktopIconView.model.count() ? true : false
                //                    }
                //                }
                //            }

                //            Component.onCompleted: {
                //                 if (false === desktopIconView.initDesktop) {
                //                    //initanim.start()
                //                 }
                //                 desktopIconView.addItemPosition(model.uri, model.position.x, model.position.y)
                //            }

                Behavior on x {
                    enabled: initDesktop && !isDraging
                    id: xTransBehavior
                    NumberAnimation {
                        easing.type: Easing.Bezier;
                        easing.bezierCurve: [0.25, 0.1, 0.38, 1];
                        duration: 300
                        onFinished: {
                            window.sorting = false
                        }
                    }
                }
                Behavior on y {
                    enabled: initDesktop && !isDraging
                    id: yTransBehavior
                    NumberAnimation {
                        easing.type: Easing.Bezier;
                        easing.bezierCurve: [0.25, 0.1, 0.38, 1];
                        duration: 300
                    }
                }

                NumberAnimation {
                    id: anim
                    target: dele
                    property: "x"
                    from: dragx
                    to: dragtox
                    duration: 180
                    easing.type: Easing.Bezier
                    easing.bezierCurve:  [0.25, 0.1, 0.38, 1] // 标准缓入缓出贝塞尔曲线
                    // 也可以使用预设的缓入缓出类型
                    // easing.type: Easing.InOutQuad
                }
                NumberAnimation {
                    id: animy
                    target: dele
                    property: "y"
                    from: dragy
                    to: dragtoy
                    duration: 180
                    easing.type: Easing.Bezier
                    easing.bezierCurve:  [0.25, 0.1, 0.38, 1] // 标准缓入缓出贝塞尔曲线
                    // 也可以使用预设的缓入缓出类型
                    // easing.type: Easing.InOutQuad
                }
                //            NumberAnimation {
                //                id: animaresort
                //                target: dele
                //                property: "x"
                //                from:desktopIconView.transtogridposx(desktopIconView.preitemsPos[dele.uri].x*dele.width,dele.width)
                //                to: dele.x
                //                duration: 300
                //                easing.type: Easing.Bezier
                //                easing.bezierCurve: [0.25, 0.1, 0.38, 1] // 标准缓入缓出贝塞尔曲线
                //                // 也可以使用预设的缓入缓出类型
                //                // easing.type: Easing.InOutQuad
                //                onFinished: {
                //                    addItemPosition(dele.uri, dele.x, dele.y)
                //                }
                //            }
                //            NumberAnimation {
                //                id: animaresorty
                //                target: dele
                //                property: "y"
                //                from: desktopIconView.transtogridposy(desktopIconView.preitemsPos[dele.uri].y*dele.height,dele.height)
                //                to: dele.y
                //                duration: 300
                //                easing.type: Easing.Bezier
                //                easing.bezierCurve: [0.25, 0.1, 0.38, 1] // 标准缓入缓出贝塞尔曲线
                //                // 也可以使用预设的缓入缓出类型
                //                // easing.type: Easing.InOutQuad
                //            }

                DelegateIcon {
                    id: viewIconRect
                    width:  controller.iconSize.width
                    height:  controller.iconSize.height
                    icon : model.decoration
                    emblems: model.emblems
                    emblemSize: controller.emblemSize
                }

                DelegateText {
                    id: delegateText
                    anchors.horizontalCenter: viewIconRect.horizontalCenter
                    anchors.top: viewIconRect.bottom
                    display: dele.display  // 显示的文本内容
                    editing: false  // 是否处于编辑状态
                    showFullText: ismultifilemode || isBoxSelection ? false : true
                    selected: parent.selected
                    isDir: model.isDir
                    isSoftLink: model.softLink
                }

                Connections {
                    target: delegateText
                    function onEditFinish(selections, newText) {
                        selectionModel.clearSelection();
                        controller.rename(selections, newText)
                        desktopIconView.currentEditIndex = -1
                    }
                }

                MouseArea {
                    id: gridViewDelegateMouseArea
                    z: 8
                    acceptedButtons: Qt.LeftButton | Qt.RightButton
                    anchors.fill: parent
                    propagateComposedEvents: true
                    hoverEnabled: true
                    property point pressPos1: Qt.point(0, 0)
                    property point releasepos: Qt.point(0, 0)
                    property Timer editTriggerTimer: Timer {
                        id: editTriggerTimer
                        interval: Qt.styleHints.mouseDoubleClickInterval
                    }
                    property Timer displayTooltipTimer: Timer  {
                        id: displayTooltipTimer
                        interval: 500
                        repeat: false
                        onTriggered: displayTooltips.visible = true
                    }
                    onEntered: (mouse) => {
                                   var globalPos = mapToItem(null, mouseX, mouseY)
                                   displayTooltips.x = globalPos.x
                                   displayTooltips.y = globalPos.y
                                   displayTooltips.z = parent.z + 2
                                   displayTooltips.tooltipText = parent.tooltip
                                   displayTooltipTimer.start()
                               }
                    onExited: {
                        displayTooltipTimer.stop()
                        displayTooltips.visible = false
                    }
                    onPressed: (mouse) => {
                                   console.log("mouse onPressed on delegate, boxselection:",isBoxSelection,dele.selected);
                                   EventBus.setCurrentScreenId(desktopIconView.currentScreentId)
                                   if (mouseX < 0 || mouseX > width || mouseY < 0 || mouseY > height) {
                                       mouse.accepted = false
                                       return;
                                   }
                                   var shiftKeyPressed = mouse.modifiers & Qt.ShiftModifier;
                                   var controlKeyPressed = mouse.modifiers & Qt.ControlModifier;
                                   if ( shiftKeyPressed || controlKeyPressed || (isBoxSelection && dele.selected)) {
                                       console.log("Ctrl or shift key is pressed!");
                                       ismultifilemode = true
                                       if (shiftKeyPressed) {
                                           var startIndex = selectionModel.firstIndex();
                                           selectionModel.selectBetween(startIndex, index)
                                       }
                                   }
                                   console.log("mouse onPressed ismultifilemode",ismultifilemode);
                                   if (mouse.button === Qt.RightButton) {
                                       if (!ismultifilemode) {
                                           selectionModel.clearSelection()
                                       }
                                       var globalPos = mapToGlobal(mouse.x, mouse.y);
                                       selectionModel.updateSelection(index, true)
                                       var selections = selectionModel.getSelections();
                                       console.log("open menu", globalPos, selections, mouse.x, mouse.y)
                                       Qt.callLater(() => controller.showMenu(selections, globalPos))
                                       mouse.accepted = true  // 阻止事件继续传递
                                       return
                                   }

                                   if (!ismultifilemode) {
                                       if (delegateText.editing) {
                                           console.log("editing........")
                                           mouse.accepted = false
                                           return
                                       }
                                       desktopIconView.closeEdit()
                                       if (dele.selected && editTriggerTimer.running) {
                                           onDoubleClicked()
                                       } else if (dele.selected && !editTriggerTimer.running) {
                                           controller.editUris(selectionModel.getSelections());
                                       } else if (!dele.selected){
                                           editTriggerTimer.start()
                                       }
                                       selectionModel.clearSelection()
                                   }

                                   pressPos1 = Qt.point(mouseX, mouseY)
                                   var realBounds = dragIconView.calculateContentBounds()
                                   dragIconView.offsetToBoundsTopLeftX = dele.x
                                   dragIconView.offsetToBoundsTopLeftY = dele.y
                                   if (controlKeyPressed && dele.selected) {
                                       dele.selected = false;
                                   } else {
                                       displayTooltipTimer.stop()
                                       displayTooltips.visible = false
                                       selectionModel.updateSelection(index, true)
                                   }
                               }
                    onReleased: {
                        console.log("mouse released");
                        isBoxSelection = false;
                        var shiftKeyPressed = mouse.modifiers & Qt.ShiftModifier;
                        var controlKeyPressed = mouse.modifiers & Qt.ControlModifier;
                        if (!(shiftKeyPressed || controlKeyPressed) && desktopIconView.currentEditIndex === -1) {
                            ismultifilemode = false
                            selectionModel.selectCurrent(index)
                        }
                    }
                    onClicked: {
                        mouse.accepted = true
                    }
                    onDoubleClicked: {
                        var item = desktopIconView.itemAt(index)
                        controller.openFileByUri(item.uri);
                        console.log("on double clicked", item.uri)
                        mouse.accepted = true
                    }

                    drag.target: dragIconView
                    drag.onActiveChanged: {
                        console.log("mouse area drag active", drag.active)
                        if (drag.active) {
                            desktopIconView.closeEdit()
                            // hotspot的计算要在bounds更新之后
                            dragIconView.Drag.source =  dele
                            dele.delegateGlobalPos = mapToItem(null, 0, 0)
                            dragIconView.Drag.mimeData = {
                                "text/uri-list": selectionModel.getSelections()
                                .map(uri => uri.trim())  // 去除每个URI两端的空格
                                .filter(uri => uri !== "")  // 过滤空URI
                                .join("\r\n"),
                                "peony-qt/encoded-uris": selectionModel.getSelections()
                                .filter(uri => uri.trim() !== "")
                                .join(" "),
                                "peony-qt/pos-list": selectionModel.getSelectionsPositions()
                                .map(point => `${point.x},${point.y}`) // 将QPoint转为"x,y"格式
                                .join(" "), // 用空格分隔多个坐标
                                "text/ukui_desktop": true
                            };

                            // grabToImage不会立即返回所以在回调中处理
                            var items = selectionModel.getSelectedItems()
                            dragitemModel.clear()
                            for(var i = 0;i < items.length; i++){
                                dragitemModel.append(items[i])
                            }
                            // 设置hotSpot为鼠标在按下相对于选中（bounds）左上角区域的位置
                            dragIconView.Drag.hotSpot.x = gridViewDelegateMouseArea.pressPos1.x + dragIconView.offsetToBoundsTopLeftX - dragIconView.bounds.x
                            dragIconView.Drag.hotSpot.y = gridViewDelegateMouseArea.pressPos1.y + dragIconView.offsetToBoundsTopLeftY - dragIconView.bounds.y
                            dragIconView.grabToImage(function(result) {
                                dragIconView.Drag.imageSource = result.url
                                dragIconView.Drag.active = true
                            })
                        } else {
                            dragIconView.Drag.active = false
                            dragIconView.x = 0
                            dragIconView.y = 0
                            pressPos1 = Qt.point(0, 0)
                            dragIconView.offsetToBoundsTopLeftX = 0
                            dragIconView.offsetToBoundsTopLeftY = 0
                            console.log("更新当前视图")
                            desktopIconView.model.invalidateModel()
                        }
                    }
                }
            }
            function addItemPosition(uri, x, y) {
                // 创建包含URI和Point的对象
                var newItem = {
                    uri: uri,
                    position: Qt.point(x, y) // 使用Qt.point创建二维点
                }
                itemsPos.push(newItem) // 添加到列表
                preitemsPos[uri] = Qt.point(x, y)
            }

            Component.onCompleted: {
                for (var i = 0; i < desktopIconView.count; i++) {
                    var item = desktopIconView.itemAt(i);
                    if (item) {
                        addItemPosition(item.uri, item.pos.x, item.pos.y);
                        console.log("44444444444444444", item.uri, item.pos.x, item.pos.y)
                    }
                }

                // 监听变化
                globalSettings.onValueChanged.connect(function(key, value) {
                    console.log("Setting changed:", key, "=", value)
                    if (key === "desktopUseAutoLayout") {
                        autoArrange = globalSettings.useAutoLayout();
                    } else if (key === "defaultDesktopZoomLevel"){
                        for (var i = 0; i < desktopIconView.count; i++) {
                            var item = desktopIconView.itemAt(i);
                            if (item) {
                                item.x = item.pos.x*desktopIconView.gridWidth
                                item.y = item.pos.y*desktopIconView.gridHeight
                            }
                        }
                    }
                })
            }
        }

        //框选矩形
        Rectangle {
            id: selectionRect
            visible: false
            color: "lightblue"
            opacity: 0.3
            border.color: "blue"
            border.width: 1
            z: 999
        }
        // 框选功能的 MouseArea
        MouseArea {
            anchors.fill: parent
            z:-1
            // 初始化选择框
            acceptedButtons: Qt.LeftButton | Qt.RightButton  // 明确接受右键
            onPressed: (mouse) => {
                           console.log("mouse onPressed on desktop area");
                           EventBus.setCurrentScreenId(desktopIconView.currentScreentId)
                           var shiftKeyPressed = mouse.modifiers & Qt.ShiftModifier;
                           var controlKeyPressed = mouse.modifiers & Qt.ControlModifier;
                           if (shiftKeyPressed || controlKeyPressed) {
                               console.log("Ctrl or Shift key is pressed!");
                               ismultifilemode = true
                           } else {
                               var index = desktopIconView.indexAt(mouseX, mouseY)
                               var item = desktopIconView.itemAt(index)
                               if (item) {
                                   if (!(mouseX >= item.x && mouseX <= item.x + item.width &&
                                         mouseY >= item.y && mouseY <= item.y + item.height)) {
                                       index = -1
                                   }
                               }
                               if (index === -1) {
                                   selectionModel.clearSelection();
                               }
                           }
                           console.log("drop to current index:", index)
                           if (mouse.button === Qt.LeftButton) {
                               pressPos = Qt.point(mouseX, mouseY)
                               selectionRect.x = mouseX
                               selectionRect.y = mouseY
                               selectionRect.width = 0
                               selectionRect.height = 0
                               selectionRect.visible = true
                               desktopIconView.closeEdit()
                           } /*else if (mouse.button === Qt.RightButton) {
                selectionRect.visible = false
                selectionRect.width = 0
                selectionRect.height = 0
                var pos = mapToGlobal(mouse.x, mouse.y)
                var selections = selectionModel.getSelections();
                controller.showMenu(selections, pos)
                mouse.accepted = true
            }*/
                       }
            onClicked: {
                if (mouse.button === Qt.RightButton) {
                    desktopIconView.closeEdit()
                    if (selectionRect.width == 0 || selectionRect.height == 0) {
                        selectionModel.clearSelection()
                    }
                    var i = desktopIconView.indexAt(mouse.x, mouse.y)
                    if (i !== -1) {
                        selectionModel.updateSelection(i, true)
                    }
                    pressPos = Qt.point(-1, -1)
                    selectionRect.visible = false
                    selectionRect.width = 0
                    selectionRect.height = 0
                    var pos = mapToGlobal(mouse.x, mouse.y)
                    var selections = selectionModel.getSelections();
                    Qt.callLater(() => controller.showMenu(selections, pos))
                    isBoxSelection = false;
                    mouse.accepted = true
                    return
                }
            }
            // 实时更新选择框尺寸
            onPositionChanged: {
                if (mouse.button === Qt.RightButton) {
                    desktopIconView.closeEdit()
                    isBoxSelection = false;
                    pressPos = Qt.point(-1, -1)
                    selectionRect.visible = false
                    selectionRect.width = 0
                    selectionRect.height = 0
                    mouse.accepted = true
                    return
                }
                if (pressPos.x === -1 && pressPos.y === -1) {
                    mouse.accepted = true
                    return
                }

                const newX = Math.min(mouseX, pressPos.x)
                const newY = Math.min(mouseY, pressPos.y)
                const newWidth = Math.abs(mouseX - pressPos.x)
                const newHeight = Math.abs(mouseY - pressPos.y)

                selectionRect.x = newX
                selectionRect.y = newY
                selectionRect.width = newWidth
                selectionRect.height = newHeight

                for (var i = 0; i < desktopIconView.count; i++) {
                    const item = desktopIconView.itemAt(i)
                    if (item && item.visible) {
                        const itemRect = Qt.rect(item.x, item.y,item.width,item.height)
                        // 检测相交
                        var tmpSelectionRect = Qt.rect(selectionRect.x, selectionRect.y, selectionRect.width, selectionRect.height);
                        const isSelected = rectanglesIntersect(itemRect, tmpSelectionRect)
                        item.selected = isSelected
                        if (isSelected) isBoxSelection = true
                        selectionModel.updateSelection(i, isSelected)
                    }
                }
            }

            function rectanglesIntersect(rectA, rectB) {
                return (rectA.x < rectB.x + rectB.width &&
                        rectA.x + rectA.width > rectB.x &&
                        rectA.y < rectB.y + rectB.height &&
                        rectA.y + rectA.height > rectB.y)
            }
            // 鼠标释放时校验选中项
            onReleased: {
                if (mouse.button === Qt.RightButton) {
                    desktopIconView.closeEdit()
                    selectionRect.visible = false
                    selectionRect.width = 0
                    selectionRect.height = 0
                    pressPos = Qt.point(-1, -1)
                    mouse.accepted = true
                    return
                }
                // 遍历所有Repeater项
                if (selectionRect.width == 0 || selectionRect.height == 0) {
                    isBoxSelection = false;
                    return
                }
                for (var i = 0; i < desktopIconView.count; i++) {
                    const item = desktopIconView.itemAt(i)
                    if (item && item.visible) {
                        const itemRect = Qt.rect(item.x, item.y,item.width,item.height)
                        // 检测相交
                        var tmpSelectionRect = Qt.rect(selectionRect.x, selectionRect.y, selectionRect.width, selectionRect.height);
                        const isSelected = rectanglesIntersect(itemRect, tmpSelectionRect)
                        if (isSelected) isBoxSelection = true
                        item.selected = isSelected || (ismultifilemode && item.selected)
                        selectionModel.updateSelection(i, item.selected)
                    }
                }
                selectionRect.visible = false
                selectionRect.width = 0
                selectionRect.height = 0
            }
        }

        DropArea {
            anchors.fill: parent
            visible: true
            onEntered: {
                if (!acceptDrop) {
                    drag.accepted = false;
                }
            }

            onExited: {
                dropIndicator.visible = false;
            }

            onPositionChanged: (mouse) => {
                                   drag.acceptedAction = (drag.modifiers & Qt.ControlModifier)
                                   ? Qt.CopyAction
                                   : Qt.MoveAction;
                                   var currentPos = Qt.point(mouse.x, mouse.y)
                                   desktopIconView.onUpdateDropIndicator(currentPos)
                               }

            onDropped: {
                // 1. 解析拖拽文件路径
                var uriString = drop.getDataAsString("text/uri-list");
                var uris = uriString.split(/\r?\n/)
                .map(uri => uri.trim())
                .filter(uri => uri !== "");
                console.log("drop uris:", uris)
                if (uris.length === 0) {
                    return;
                }

                var globalDropPos = Qt.point(drop.x, drop.y);
                var y = drop.y
                if (dropIndicator.visible && y < dropIndicator.y) {
                    y = y + desktopIconView.gridHeight
                }
                var index = desktopIconView.indexAt(drop.x, y)
                console.log("drop to current index:", index)
                if (!(drag.source && drag.source.isFromDesktopView)) {
                    console.log("其他应用拖拽", uris, desktopIconView.count)
                    dropIndicator.visible = false
                    desktopIconView.model.addFile(uris, drop.proposedAction, index);
                    return;
                }

                // 2. 计算热区点的全局坐标（delegate初始位置 + 热区偏移）
                var hotspotGlobalX = drop.source.delegateGlobalPos.x + drop.source.pressPos1.x
                var hotspotGlobalY = drop.source.delegateGlobalPos.y + drop.source.pressPos1.y

                // 3. 计算释放点相对于热区的偏移（网格单位）
                var viewLocalPos = desktopIconView.mapFromItem(null, globalDropPos.x, globalDropPos.y);
                var offGridX = Math.floor(viewLocalPos.x / desktopIconView.gridWidth) - Math.floor(hotspotGlobalX / desktopIconView.gridWidth);
                var offGridY = Math.floor(viewLocalPos.y / desktopIconView.gridHeight) - Math.floor(hotspotGlobalY / desktopIconView.gridHeight);

                console.log("drop parameter", drop.x, drop.y, globalDropPos.x,globalDropPos.y, offGridX, offGridY,drop.source.delegateGlobalPos,drop.source.pressPos1)
                console.log("drop parameter1", dragIconView.Drag.hotSpot.x,dragIconView.Drag.hotSpot.y, viewLocalPos,hotspotGlobalX, hotspotGlobalY)

                var dragSource = drag.source
                if (drag.source && drag.source.isFromDesktopView) {
                    var onItem =  globalSettings.useAutoLayout() && -1 !== index? !dropIndicator.visible : -1 !== index
                    dropIndicator.visible = false
                    // drop事件从c++更新图标网格时，需要屏蔽x和y默认的behavior，使用拖拽动效
                    window.isDraging = true
                    desktopIconView.model.updateIconPositions(uris, drop.proposedAction, index, Qt.point(offGridX, offGridY), onItem);
                    window.isDraging = false
                }
                var offx = viewLocalPos.x - hotspotGlobalX;
                var offy = viewLocalPos.y - hotspotGlobalY;

                var posString = drop.getDataAsString("peony-qt/pos-list");

                var points = posString.split(" ")
                .map(pointStr => {
                         var xy = pointStr.split(",");
                         return xy.length === 2 ?
                             Qt.point(parseInt(xy[0]), parseInt(xy[1])) :
                             null;
                     })
                .filter(point => point !== null);

                points.forEach((pt, index) => {
                                   console.log(`Item ${index}: (${pt.x}, ${pt.y})`);
                               });

                for (var i = 0; i < desktopIconView.count; i++) {
                    const item = desktopIconView.itemAt(i)
                    if (item && item.visible && uris.includes(item.uri)) {
                        var j = uris.indexOf(item.uri)
                        if (j === -1) {
                            continue
                        }
                        item.visible = false
                        item.dragx = points[j].x*desktopIconView.gridWidth + offx
                        item.dragy= points[j].y*desktopIconView.gridHeight + offy
                        console.log("anim start pos", j, item.dragx, item.dragy, offx, offy, item.x, item.y)
                        var gridposx = desktopIconView.transtogridposx(item.pos.x*desktopIconView.gridWidth,desktopIconView.gridWidth)
                        var gridposy = desktopIconView.transtogridposy(item.pos.y*desktopIconView.gridHeight,desktopIconView.gridHeight)
                        item.dragtox = gridposx
                        item.dragtoy = gridposy
                        console.log("anim end pos", item.dragtox, item.dragtoy)
                        item.animItem.start()
                        item.animyItem.start()
                        item.visible = true
                    }
                }
                drop.source.pressPos1.x = 0
                drop.source.pressPos1.y = 0
                selectionModel.clearSelection();
            }
        }

        Connections {
            target: myProxyModel
            function onViewUpdated() {
                if (!initDesktop) {
                    initDesktop = true
                }
            }
        }

        Connections{
            target: myProxyModel
            function onLayoutUpdate() {
                console.log("model layout updated")
                layoutchange()
            }
        }

        Connections{
            target: controller
            function onEdit (uri) {
                for (var i = 0; i < desktopIconView.count; i++) {
                    var item = desktopIconView.itemAt(i);
                    if (item && item.uri === uri && item.delegateText) {
                        console.log("Editing item at index:", i);
                        desktopIconView.currentEditIndex = i;
                        item.delegateText.selectedUris = selectionModel.getSelections();
                        if (!item.delegateText.selectedUris.includes(uri)) {
                            item.delegateText.selectedUris.push(uri)
                        }
                        item.delegateText.forceActiveFocus();
                    }
                }
            }
            function onChangeOpacity(opacity) {
                for (var i = 0; i < desktopIconView.count; i++) {
                    var item = desktopIconView.itemAt(i);
                    if (item) {
                        if (item.selected) {
                            item.opacity = opacity
                        } else {
                            item.opacity = 1
                        }
                    }
                }
            }

            function onSelectedUrisChanged (uriList) {
                for (var i = 0; i < desktopIconView.count; i++) {
                    var item = desktopIconView.itemAt(i);
                    if (item) {
                        if (uriList.includes(item.uri)) {
                            selectionModel.updateSelection(i, true)
                        } else {
                            selectionModel.updateSelection(i, false)
                        }
                    }
                }
            }
        }

        Keys.onPressed: (event) => {
                            var itemIndex
                            switch(event.key) {
                                case Qt.Key_Home:
                                selectionModel.selectHome();
                                event.accepted = true;
                                break;
                                case Qt.Key_End:
                                selectionModel.selectEnd();
                                event.accepted = true;
                                break;
                                case Qt.Key_Up:
                                selectionModel.selectPrevious();
                                event.accepted = true;
                                break;
                                case Qt.Key_Down:
                                selectionModel.selectNext();
                                event.accepted = true;
                                break;
                                case Qt.Key_Left:
                                selectionModel.selectLeft();
                                event.accepted = true;
                                break;
                                case Qt.Key_Right:
                                selectionModel.selectRight();
                                event.accepted = true;
                                break;
                                case Qt.Key_Enter:
                                case Qt.Key_Return:
                                var uris = selectionModel.getSelections()
                                controller.openFileByUri(uris);
                                event.accepted = true;
                                break;
                            }
                        }
    }
}
